(load "~/quicklisp/setup.lisp") ; need to load quicklisp
(ql:quickload '(:cl-raylib :cl-bcrypt :websocket-driver-client :alexandria :com.inuoe.jzon))

(defpackage :mmo-client
  (:use :common-lisp :cl-raylib :raylib :3d-vectors :websocket-driver-client :alexandria :com.inuoe.jzon))

(in-package :mmo-client)

(defvar *client* (wsd:make-client "ws://localhost:12345"))

(wsd:start-connection *client*)
(wsd:on :message *client*
        (lambda (message)
          (format t "~A~%" message)))

(let* ((screen-width 800)
       (screen-height 450)
       (username "zongor")
       (title "mmo client - common lisp")
       (camera (make-camera3d :position (vec 4.0 2.0 4.0)
                              :target (vec 0.0 0.5 0.0)
                              :up (vec 0.0 1.0 0.0)
                              :fovy 60.0
                              :projection :camera-perspective))
       (cube-screen-pos (vec 0.0 0.0)))

  (with-window (screen-width screen-height title)
   ;; (disable-cursor)
    (set-target-fps 60) ; Set our game to run at 60 FPS

    (wsd:send *client* (com.inuoe.jzon:stringify (list :login username)))
    (loop
      until (window-should-close) ; detect window close button or ESC key
      do
	 (update-camera camera :camera-third-person)

         (when (is-key-down 87)
	     (wsd:send *client* (com.inuoe.jzon:stringify (list :move (camera3d-target camera)))))
         (when (is-key-down 65)
	     (wsd:send *client* (com.inuoe.jzon:stringify (list :move (camera3d-target camera)))))
         (when (is-key-down 83)
	     (wsd:send *client* (com.inuoe.jzon:stringify (list :move (camera3d-target camera)))))
         (when (is-key-down 68)
	     (wsd:send *client* (com.inuoe.jzon:stringify (list :move (camera3d-target camera)))))

	 (setf cube-screen-pos (get-world-to-screen (v+ (camera3d-target camera) (vec 0 1.0 0)) camera))
	 (with-drawing
	     (clear-background :raywhite)
	   (with-mode-3d (camera)
	     (draw-cube (camera3d-target camera) 1.0 1.0 1.0 :red)
	     (draw-cube-wires (camera3d-target camera) 1.0 1.0 1.0 :maroon)
	     (draw-grid 20 1.0))
	   (draw-text username (- (floor (vx cube-screen-pos)) (floor (measure-text username 20) 2)) 
		      (floor (vy cube-screen-pos) ) 20 :black))))
  (wsd:send *client* (com.inuoe.jzon:stringify (list :logout username))))

(wsd:close-connection *client*)
